package com.sun.jndi.ldap;

import java.io.UnsupportedEncodingException;

public final class BerDecoder extends Ber
{
    private int origOffset;
    
    public BerDecoder(byte[] buf, int origOffset, int bufsize) {
        this.buf = buf;
        this.bufsize = bufsize;
        this.origOffset = origOffset;
        this.reset();
    }
    
    public void reset() {
        this.offset = this.origOffset;
    }
    
    public int getParsePosition() {
        return this.offset;
    }
    
    public int parseLength() throws Ber.DecodeException {
        int byte1 = this.parseByte();
        if ((byte1 & 0x80) == 0x80) {
            int n = byte1 & 0x7F;
            if (n == 0) {
                throw new Ber.DecodeException("Indefinite length not supported");
            }
            if (n > 4) {
                throw new Ber.DecodeException("encoding too long");
            }
            if (this.bufsize - this.offset < n) {
                throw new Ber.DecodeException("Insufficient data");
            }
            int n2 = 0;
            for (int i = 0; i < n; ++i) {
                n2 = (n2 << 8) + (this.buf[this.offset++] & 0xFF);
            }
            if (n2 < 0) {
                throw new Ber.DecodeException("Invalid length bytes");
            }
            return n2;
        }
        else {
            return byte1;
        }
    }
    
    public int parseSeq(int[] array) throws Ber.DecodeException {
        int byte1 = this.parseByte();
        int length = this.parseLength();
        if (array != null) {
            array[0] = length;
        }
        return byte1;
    }
    
    void seek(int n) throws Ber.DecodeException {
        if (this.offset + n > this.bufsize || this.offset + n < 0) {
            throw new Ber.DecodeException("array index out of bounds");
        }
        this.offset += n;
        return;
    }
    
    public int parseByte() throws Ber.DecodeException {
        if (this.bufsize - this.offset < 1) {
            throw new Ber.DecodeException("Insufficient data");
        }
        return this.buf[this.offset++] & 0xFF;
    }
    
    public int peekByte() throws Ber.DecodeException {
        if (this.bufsize - this.offset < 1) {
            throw new Ber.DecodeException("Insufficient data");
        }
        return this.buf[this.offset] & 0xFF;
    }
    
    public boolean parseBoolean() throws Ber.DecodeException {
        return this.parseIntWithTag(1) != 0;
    }
    
    public int parseEnumeration() throws Ber.DecodeException {
        return this.parseIntWithTag(10);
    }
    
    public int parseInt() throws Ber.DecodeException {
        return this.parseIntWithTag(2);
    }
    
    private int parseIntWithTag(int i) throws Ber.DecodeException {
        if (this.parseByte() != i) {
            String string;
            if (this.offset > 0) {
                string = Integer.toString(this.buf[this.offset - 1] & 0xFF);
            }
            else {
                string = "Empty tag";
            }
            throw new Ber.DecodeException("Encountered ASN.1 tag " + string + " (expected tag " + Integer.toString(i) + ")");
        }
        else {
            int length = this.parseLength();
            if (length > 4) {
                throw new Ber.DecodeException("INTEGER too long");
            }
            if (length > this.bufsize - this.offset) {
                throw new Ber.DecodeException("Insufficient data");
            }
            int n = this.buf[this.offset++];
            int n2 = n & 0x7F;
            for (int j = 1; j < length; ++j) {
                n2 = (n2 << 8 | (this.buf[this.offset++] & 0xFF));
            }
            if ((n & 0x80) == 0x80) {
                n2 = -n2;
            }
            return n2;
        }
    }
    
    public String parseString(boolean b) throws Ber.DecodeException {
        return this.parseStringWithTag(4, b, (int[])null);
    }
    
    public String parseStringWithTag(int i, boolean b, int[] array) throws Ber.DecodeException {
        int offset = this.offset;
        int byte1;
        if ((byte1 = this.parseByte()) != i) {
            throw new Ber.DecodeException("Encountered ASN.1 tag " + Integer.toString((int)(byte)byte1) + " (expected tag " + i + ")");
        }
        int length = this.parseLength();
        if (length > this.bufsize - this.offset) {
            throw new Ber.DecodeException("Insufficient data");
        }
        String s;
        if (length == 0) {
            s = "";
        }
        else {
            byte[] array2 = new byte[length];
            System.arraycopy(this.buf, this.offset, array2, 0, length);
            if (b) {
                try {
                    s = new String(array2, "UTF8");
                }
                catch (UnsupportedEncodingException ex2) {
                    throw new Ber.DecodeException("UTF8 not available on platform");
                }
            }
            else {
                try {
                    s = new String(array2, "8859_1");
                }
                catch (UnsupportedEncodingException ex) {
                    throw new Ber.DecodeException("8859_1 not available on platform");
                }
            }
            this.offset += length;
        }
        if (array != null) {
            array[0] = this.offset - offset;
            return s;
        }
        return s;
    }
    
    public byte[] parseOctetString(int i, int[] array) throws Ber.DecodeException {
        int offset = this.offset;
        int byte1;
        if ((byte1 = this.parseByte()) != i) {
            throw new Ber.DecodeException("Encountered ASN.1 tag " + Integer.toString(byte1) + " (expected tag " + Integer.toString(i) + ")");
        }
        int length = this.parseLength();
        if (length > this.bufsize - this.offset) {
            throw new Ber.DecodeException("Insufficient data");
        }
        byte[] array2 = new byte[length];
        if (length > 0) {
            System.arraycopy(this.buf, this.offset, array2, 0, length);
            this.offset += length;
        }
        if (array != null) {
            array[0] = this.offset - offset;
            return array2;
        }
        return array2;
    }
    
    public int bytesLeft() {
        return this.bufsize - this.offset;
    }
}
